@@ -29,8 +29,8 @@ void SetDrawtoField_MM(int mode)
   // for convenience, absolute screen position to centered level playfield
   cSX = SX + dSX;
   cSY = SY + dSY;
-  cSX2 = SX + dSX + 2;	// including playfield border
-  cSY2 = SY + dSY + 2;	// including playfield border
+  cSX2 = SX + dSX + 2;		// including half laser line size
+  cSY2 = SY + dSY + 2;		// including half laser line size
 
   if (mode == DRAW_TO_BACKBUFFER)
   {
@@ -41,9 +41,16 @@ void SetDrawtoField_MM(int mode)
   SetTileCursorSXSY(cSX, cSY);
 }
 
+void BackToFront_MM(void)
+{
+  BlitScreenToBitmap_MM(backbuffer);
+
+  BackToFront();
+}
+
 void ClearWindow(void)
 {
-  ClearRectangle(backbuffer, REAL_SX, REAL_SY, FULL_SXSIZE, FULL_SYSIZE);
+  ClearRectangle(drawto_mm, REAL_SX, REAL_SY, FULL_SXSIZE, FULL_SYSIZE);
 
   SetDrawtoField(DRAW_TO_BACKBUFFER);
   SetDrawtoField_MM(DRAW_TO_BACKBUFFER);
@@ -58,14 +65,14 @@ void DrawGraphicAnimation_MM(int x, int y, int graphic, int frame)
 
   getGraphicSource(graphic, frame, &bitmap, &src_x, &src_y);
 
-  BlitBitmap(bitmap, drawto_field, src_x, src_y, TILEX, TILEY,
+  BlitBitmap(bitmap, drawto_mm, src_x, src_y, TILEX, TILEY,
 	     cFX + x * TILEX, cFY + y * TILEY);
 }
 
 void DrawGraphic_MM(int x, int y, int graphic)
 {
 #if DEBUG
-  if (!IN_SCR_FIELD(x,y))
+  if (!IN_SCR_FIELD(x, y))
   {
     Debug("game:mm:DrawGraphic_MM", "x = %d, y = %d, graphic = %d",
 	  x, y, graphic);
@@ -95,9 +102,9 @@ void DrawGraphicExt_MM(DrawBuffer *d, int x, int y, int graphic)
 void DrawGraphicThruMask_MM(int x, int y, int graphic, int frame)
 {
 #if DEBUG
-  if (!IN_SCR_FIELD(x,y))
+  if (!IN_SCR_FIELD(x, y))
   {
-    Debug("game:mm:DrawGraphicThruMask_MM", "x = %d,y = %d, graphic = %d",
+    Debug("game:mm:DrawGraphicThruMask_MM", "x = %d, y = %d, graphic = %d",
 	  x, y, graphic);
     Debug("game:mm:DrawGraphicThruMask_MM", "This should never happen!");
 
@@ -105,10 +112,10 @@ void DrawGraphicThruMask_MM(int x, int y, int graphic, int frame)
   }
 #endif
 
-  DrawGraphicThruMaskExt_MM(drawto_field, cFX + x * TILEX, cFY + y * TILEY,
+  DrawGraphicThruMaskExt_MM(drawto_mm, cFX + x * TILEX, cFY + y * TILEY,
 			    graphic, frame);
 
-  MarkTileDirty(x,y);
+  MarkTileDirty(x, y);
 }
 
 void DrawGraphicThruMaskExt_MM(DrawBuffer *d, int dest_x, int dest_y,
@@ -127,19 +134,12 @@ void DrawGraphicThruMaskExt_MM(DrawBuffer *d, int dest_x, int dest_y,
 
 void DrawMiniGraphic_MM(int x, int y, int graphic)
 {
-  DrawMiniGraphicExt_MM(drawto, cSX + x * MINI_TILEX, cSY + y * MINI_TILEY,
+  DrawMiniGraphicExt_MM(drawto_mm, cSX + x * MINI_TILEX, cSY + y * MINI_TILEY,
 			graphic);
 
   MarkTileDirty(x / 2, y / 2);
 }
 
-#if 0
-static void getMicroGraphicSource(int graphic, Bitmap **bitmap, int *x, int *y)
-{
-  getSizedGraphicSource(graphic, 0, TILESIZE / 4, bitmap, x, y);
-}
-#endif
-
 void DrawMiniGraphicExt_MM(DrawBuffer *d, int x, int y, int graphic)
 {
   Bitmap *bitmap;
@@ -150,8 +150,8 @@ void DrawMiniGraphicExt_MM(DrawBuffer *d, int x, int y, int graphic)
   BlitBitmap(bitmap, d, src_x, src_y, MINI_TILEX, MINI_TILEY, x, y);
 }
 
-void DrawGraphicShifted_MM(int x,int y, int dx,int dy, int graphic,
-			int cut_mode, int mask_mode)
+void DrawGraphicShifted_MM(int x, int y, int dx, int dy, int graphic,
+			   int cut_mode, int mask_mode)
 {
   int width = TILEX, height = TILEY;
   int cx = 0, cy = 0;
@@ -242,7 +242,7 @@ void DrawGraphicShifted_MM(int x,int y, int dx,int dy, int graphic,
   dest_y = cFY + y * TILEY + dy;
 
 #if DEBUG
-  if (!IN_SCR_FIELD(x,y))
+  if (!IN_SCR_FIELD(x, y))
   {
     Debug("game:mm:DrawGraphicShifted_MM", "x = %d, y = %d, graphic = %d",
 	  x, y, graphic);
@@ -253,13 +253,13 @@ void DrawGraphicShifted_MM(int x,int y, int dx,int dy, int graphic,
 #endif
 
   if (mask_mode == USE_MASKING)
-    BlitBitmapMasked(src_bitmap, drawto_field,
+    BlitBitmapMasked(src_bitmap, drawto_mm,
 		     src_x, src_y, TILEX, TILEY, dest_x, dest_y);
   else
-    BlitBitmap(src_bitmap, drawto_field,
+    BlitBitmap(src_bitmap, drawto_mm,
 	       src_x, src_y, width, height, dest_x, dest_y);
 
-  MarkTileDirty(x,y);
+  MarkTileDirty(x, y);
 }
 
 void DrawScreenElementExt_MM(int x, int y, int dx, int dy, int element,
@@ -364,7 +364,24 @@ void DrawScreenField_MM(int x, int y)
 
 void DrawLevelField_MM(int x, int y)
 {
-  DrawScreenField_MM(x, y);
+  if (IN_SCR_FIELD(SCREENX(x), SCREENY(y)))
+    DrawScreenField_MM(SCREENX(x), SCREENY(y));
+  else if (IS_MOVING(x, y))
+  {
+    int newx, newy;
+
+    Moving2Blocked(x, y, &newx, &newy);
+    if (IN_SCR_FIELD(SCREENX(newx), SCREENY(newy)))
+      DrawScreenField_MM(SCREENX(newx), SCREENY(newy));
+  }
+  else if (IS_BLOCKED(x, y))
+  {
+    int oldx, oldy;
+
+    Blocked2Moving(x, y, &oldx, &oldy);
+    if (IN_SCR_FIELD(SCREENX(oldx), SCREENY(oldy)))
+      DrawScreenField_MM(SCREENX(oldx), SCREENY(oldy));
+  }
 }
 
 void DrawMiniElement_MM(int x, int y, int element)
@@ -402,7 +419,7 @@ void DrawField_MM(int x, int y)
 
 void DrawLevel_MM(void)
 {
-  int x,y;
+  int x, y;
 
   ClearWindow();
 
@@ -441,10 +458,10 @@ void DrawWallsExt_MM(int x, int y, int element, int draw_mask)
       continue;
 
     if (element & (1 << i))
-      BlitBitmap(bitmap, drawto, gx, gy, MINI_TILEX, MINI_TILEY,
+      BlitBitmap(bitmap, drawto_mm, gx, gy, MINI_TILEX, MINI_TILEY,
 		 dest_x, dest_y);
     else
-      ClearRectangle(drawto, dest_x, dest_y, MINI_TILEX, MINI_TILEY);
+      ClearRectangle(drawto_mm, dest_x, dest_y, MINI_TILEX, MINI_TILEY);
   }
 
   MarkTileDirty(x, y);
@@ -495,7 +512,7 @@ void DrawWallsAnimation_MM(int x, int y, int element, int phase, int bit_mask)
       getSizedGraphicSource(graphic, frame, MINI_TILESIZE, &bitmap,
 			    &src_x, &src_y);
 
-      BlitBitmap(bitmap, drawto, src_x, src_y, MINI_TILEX, MINI_TILEY,
+      BlitBitmap(bitmap, drawto_mm, src_x, src_y, MINI_TILEX, MINI_TILEY,
 		 dst_x, dst_y);
     }
   }
@@ -524,68 +541,18 @@ void DrawElement_MM(int x, int y, int element)
 	   laser.fuse_x == x &&
 	   laser.fuse_y == y)
     DrawGraphic_MM(x, y, IMG_MM_FUSE);
+  else if (element == EL_GRAY_BALL_ACTIVE)
+    DrawGraphic_MM(x, y, el_act2gfx(EL_GRAY_BALL, MM_ACTION_ACTIVE));
+  else if (element == EL_GRAY_BALL_OPENING)
+    DrawGraphic_MM(x, y, el_act2gfx(EL_GRAY_BALL, MM_ACTION_OPENING));
+  else if (element == EL_BOMB_ACTIVE)
+    DrawGraphic_MM(x, y, el_act2gfx(EL_BOMB, MM_ACTION_ACTIVE));
+  else if (element == EL_MINE_ACTIVE)
+    DrawGraphic_MM(x, y, el_act2gfx(EL_MINE, MM_ACTION_ACTIVE));
   else
     DrawGraphic_MM(x, y, el2gfx(element));
 }
 
-#if 0
-static void DrawMicroWalls_MM(int x, int y, int element)
-{
-  Bitmap *bitmap;
-  int graphic = el2gfx(WALL_BASE(element));
-  int gx, gy;
-  int i;
-
-  getMicroGraphicSource(graphic, &bitmap, &gx, &gy);
-
-  for (i = 0; i < 4; i++)
-  {
-    int xpos = MICROLEV_XPOS + x * MICRO_TILEX + MICRO_WALLX * (i % 2);
-    int ypos = MICROLEV_YPOS + y * MICRO_TILEY + MICRO_WALLY * (i / 2);
-
-    if (element & (1 << i))
-      BlitBitmap(bitmap, drawto, gx, gy, MICRO_WALLX, MICRO_WALLY, xpos, ypos);
-    else
-      ClearRectangle(drawto, xpos, ypos, MICRO_WALLX, MICRO_WALLY);
-  }
-}
-
-static void DrawMicroElement_MM(int x, int y, int element)
-{
-  Bitmap *bitmap;
-  int graphic = el2gfx(element);
-  int gx, gy;
-
-  if (element == EL_EMPTY)
-    return;
-
-  if (IS_WALL(element))
-  {
-    DrawMicroWalls_MM(x, y, element);
-
-    return;
-  }
-
-  getMicroGraphicSource(graphic, &bitmap, &gx, &gy);
-
-  BlitBitmap(bitmap, drawto, gx, gy, MICRO_TILEX, MICRO_TILEY,
-	     MICROLEV_XPOS + x * MICRO_TILEX, MICROLEV_YPOS + y * MICRO_TILEY);
-}
-
-static void DrawMicroLevelExt_MM(int xpos, int ypos)
-{
-  int x, y;
-
-  ClearRectangle(drawto, xpos, ypos, MICROLEV_XSIZE, MICROLEV_YSIZE);
-
-  for (x = 0; x < STD_LEV_FIELDX; x++)
-    for (y = 0; y < STD_LEV_FIELDY; y++)
-      DrawMicroElement_MM(x, y, Ur[x][y]);
-
-  redraw_mask |= REDRAW_FIELD;
-}
-#endif
-
 
 // ----------------------------------------------------------------------------
 // XSN
@@ -680,7 +647,7 @@ static int xsn_percent(void)
   int xsn_m3 = xsn_m2 + 10;
   time_t xsn_e0 = time(NULL);
   struct tm *xsn_t0 = localtime(&xsn_e0);
-  struct tm xsn_t1 = { 0,0,0, xsn_m2*3, xsn_m3/3, xsn_t0->tm_year, 0,0,-1 };
+  struct tm xsn_t1 = { 0,0,0, xsn_m2 * 3, xsn_m3 / 3, xsn_t0->tm_year, 0,0,-1 };
   time_t xsn_e1 = mktime(&xsn_t1);
   int xsn_c0 = (25 * xsn_m3) << xsn_m1;
   int xsn_c1 = (xsn_t1.tm_wday - xsn_m1) * !!xsn_t1.tm_wday;
@@ -1151,31 +1118,11 @@ void DrawTileCursor_MM(int draw_target, boolean tile_cursor_active)
 		     dst_x, dst_y);
 }
 
-#if 0
-static int REQ_in_range(int x, int y)
-{
-  if (y > DY + 249 && y < DY + 278)
-  {
-    if (x > DX + 1 && x < DX + 48)
-      return 1;
-    else if (x > DX + 51 && x < DX + 98)
-      return 2;
-  }
-
-  return 0;
-}
-#endif
-
 Pixel ReadPixel(DrawBuffer *bitmap, int x, int y)
 {
   return GetPixel(bitmap, x, y);
 }
 
-void SetRGB(unsigned int pixel,
-	    unsigned short red, unsigned short green, unsigned short blue)
-{
-}
-
 int get_base_element(int element)
 {
   if (IS_MIRROR(element))
@@ -1200,6 +1147,10 @@ int get_base_element(int element)
     return EL_DF_MIRROR_START;
   else if (IS_DF_MIRROR_AUTO(element))
     return EL_DF_MIRROR_AUTO_START;
+  else if (IS_DF_MIRROR_FIXED(element))
+    return EL_DF_MIRROR_FIXED_START;
+  else if (IS_DF_SLOPE(element))
+    return EL_DF_SLOPE_START;
   else if (IS_PACMAN(element))
     return EL_PACMAN_START;
   else if (IS_GRID_STEEL(element))
@@ -1243,7 +1194,8 @@ int get_num_elements(int element)
       IS_POLAR(element) ||
       IS_BEAMER(element) ||
       IS_DF_MIRROR(element) ||
-      IS_DF_MIRROR_AUTO(element))
+      IS_DF_MIRROR_AUTO(element) ||
+      IS_DF_MIRROR_FIXED(element))
     return 16;
   else if (IS_GRID_STEEL_FIXED(element) ||
 	   IS_GRID_WOOD_FIXED(element) ||
@@ -1257,7 +1209,8 @@ int get_num_elements(int element)
 	   IS_RECEIVER(element) ||
 	   IS_PACMAN(element) ||
 	   IS_GRID_STEEL(element) ||
-	   IS_GRID_WOOD(element))
+	   IS_GRID_WOOD(element) ||
+	   IS_DF_SLOPE(element))
     return 4;
   else
     return 1;
@@ -1272,16 +1225,135 @@ int get_rotated_element(int element, int step)
   return base_element + (element_phase + step + num_elements) % num_elements;
 }
 
-static int map_element(int element)
+static boolean has_full_rotation(int element)
+{
+  return (IS_BEAMER(element) ||
+	  IS_MCDUFFIN(element) ||
+	  IS_LASER(element) ||
+	  IS_RECEIVER(element) ||
+	  IS_PACMAN(element));
+}
+
+#define MM_FLIP_X			0
+#define MM_FLIP_Y			1
+#define MM_FLIP_XY			2
+
+static int getFlippedTileExt_MM(int element, int mode)
+{
+  if (IS_WALL(element))
+  {
+    int base = WALL_BASE(element);
+    int bits = WALL_BITS(element);
+
+    if (mode == MM_FLIP_X)
+    {
+      bits = ((bits & 1) << 1 |
+	      (bits & 2) >> 1 |
+	      (bits & 4) << 1 |
+	      (bits & 8) >> 1);
+    }
+    else if (mode == MM_FLIP_Y)
+    {
+      bits = ((bits & 1) << 2 |
+	      (bits & 2) << 2 |
+	      (bits & 4) >> 2 |
+	      (bits & 8) >> 2);
+    }
+    else if (mode == MM_FLIP_XY)
+    {
+      bits = ((bits & 1) << 0 |
+	      (bits & 2) << 1 |
+	      (bits & 4) >> 1 |
+	      (bits & 8) >> 0);
+    }
+
+    element = base | bits;
+  }
+  else
+  {
+    int base_element = get_base_element(element);
+    int num_elements = get_num_elements(element);
+    int element_phase = element - base_element;
+
+    if (IS_GRID_STEEL(element) || IS_GRID_WOOD(element))
+    {
+      if ((mode == MM_FLIP_XY && element_phase < 2) ||
+	  (mode != MM_FLIP_XY && element_phase > 1))
+	element_phase ^= 1;
+    }
+    else if (IS_DF_SLOPE(element))
+    {
+      element_phase = (mode == MM_FLIP_X  ? 5 - element_phase :
+		       mode == MM_FLIP_Y  ? 3 - element_phase :
+		       mode == MM_FLIP_XY ? 4 - element_phase :
+		       element_phase);
+    }
+    else
+    {
+      int num_elements_flip = num_elements;
+
+      if (has_full_rotation(element))
+      {
+	if (mode == MM_FLIP_X)
+	  num_elements_flip = num_elements / 2;
+	else if (mode == MM_FLIP_XY)
+	  num_elements_flip = num_elements * 3 / 4;
+      }
+      else
+      {
+	if (mode == MM_FLIP_XY)
+	  num_elements_flip = num_elements / 2;
+      }
+
+      element_phase = num_elements_flip - element_phase;
+    }
+
+    element = base_element + (element_phase + num_elements) % num_elements;
+  }
+
+  return element;
+}
+
+int getFlippedTileX_MM(int element)
+{
+  return getFlippedTileExt_MM(element, MM_FLIP_X);
+}
+
+int getFlippedTileY_MM(int element)
+{
+  return getFlippedTileExt_MM(element, MM_FLIP_Y);
+}
+
+int getFlippedTileXY_MM(int element)
+{
+  return getFlippedTileExt_MM(element, MM_FLIP_XY);
+}
+
+int map_wall_from_base_element(int element)
 {
   switch (element)
   {
-    case EL_WALL_STEEL:		return EL_STEEL_WALL;
-    case EL_WALL_WOOD:		return EL_WOODEN_WALL;
-    case EL_WALL_ICE:		return EL_ICE_WALL;
-    case EL_WALL_AMOEBA:	return EL_AMOEBA_WALL;
-    case EL_DF_WALL_STEEL:	return EL_DF_STEEL_WALL;
-    case EL_DF_WALL_WOOD:	return EL_DF_WOODEN_WALL;
+    case EL_WALL_STEEL_BASE:	return EL_WALL_STEEL;
+    case EL_WALL_WOOD_BASE:	return EL_WALL_WOOD;
+    case EL_WALL_ICE_BASE:	return EL_WALL_ICE;
+    case EL_WALL_AMOEBA_BASE:	return EL_WALL_AMOEBA;
+    case EL_DF_WALL_STEEL_BASE:	return EL_DF_WALL_STEEL;
+    case EL_DF_WALL_WOOD_BASE:	return EL_DF_WALL_WOOD;
+
+    default:			return element;
+  }
+}
+
+int map_wall_to_base_element(int element)
+{
+  switch (element)
+  {
+    case EL_WALL_STEEL:		return EL_WALL_STEEL_BASE;
+    case EL_WALL_WOOD:		return EL_WALL_WOOD_BASE;
+    case EL_WALL_ICE:		return EL_WALL_ICE_BASE;
+    case EL_WALL_AMOEBA:	return EL_WALL_AMOEBA_BASE;
+    case EL_DF_WALL_STEEL:	return EL_DF_WALL_STEEL_BASE;
+    case EL_DF_WALL_WOOD:	return EL_DF_WALL_WOOD_BASE;
 
     default:			return element;
   }
@@ -1289,7 +1361,12 @@ static int map_element(int element)
 
 int el2gfx(int element)
 {
-  return el2img_mm(map_element(element));
+  return el2img_mm(map_wall_from_base_element(element));
+}
+
+int el_act2gfx(int element, int action)
+{
+  return el_act2img_mm(map_wall_from_base_element(element), action);
 }
 
 void RedrawPlayfield_MM(void)
@@ -1300,6 +1377,6 @@ void RedrawPlayfield_MM(void)
 
 void BlitScreenToBitmap_MM(Bitmap *target_bitmap)
 {
-  BlitBitmap(drawto_field, target_bitmap,
+  BlitBitmap(drawto_mm, target_bitmap,
 	     REAL_SX, REAL_SY, FULL_SXSIZE, FULL_SYSIZE, REAL_SX, REAL_SY);
 }
